#include "serial.h"

#define	UARTBase		0xe000c000	// UART0$B$K8GDj(B
#define	UARTClock		(CPU_CLOCK / 4)	// Hz
#define	UARTSpeed		38400		// bps
#define	UARTDivisor		(UARTClock / (UARTSpeed * 16))

#define	InReg(adr)		*((volatile unsigned long *)(adr))
#define	OutReg(adr, dat)	{*((volatile unsigned long *)(adr)) = (dat);}

#define	URBR			(UARTBase + 0x00)	// R-
#define	UTHR			(UARTBase + 0x00)	// -W
#define	UIER			(UARTBase + 0x04)	// RW
#define	UIIR			(UARTBase + 0x08)	// R-
#define	UFCR			(UARTBase + 0x08)	// -W
#define	ULCR			(UARTBase + 0x0c)	// RW
#define	UMCR			(UARTBase + 0x10)	// RW
#define	ULSR			(UARTBase + 0x14)	// R-
#define	UMSR			(UARTBase + 0x18)	// R-

#define	UDLL			(UARTBase + 0x00)	// RW
#define	UDLM			(UARTBase + 0x04)	// RW

/* LPC23XX$BFH<+$N%l%8%9%?!'%T%s$*$h$S>JEENO$N@_Dj(B */
#define	PCONP			((volatile unsigned long *)0xe01fc0c4)
#define	PINSEL0			((volatile unsigned long *)0xe002c000)

/* $B%G%P%$%9=i4|2=(B */
int serial_init(int index)
{
  *PCONP |= (1 << 3);		// UART0$B5/F0(B
  *PINSEL0 = (*PINSEL0 & ~(0x0f << 4)) | (0x05 << 4);
				// TXD0/RXD0$B%T%s$rM-8z$K$9$k(B

  OutReg(UIER, 0x00);		// $B3d$j9~$_H/@8$r6X;_(B
  while (!(InReg(ULSR) & 0x40));// $BAw?.%G!<%?$,$"$C$?>l9g$OAw?.40N;$^$GBT$D(B
  OutReg(ULCR, 0x80);		// $BJ,<~Hf%l%8%9%?$K@Z$jBX$((B
  OutReg(UDLM, 0xff);		// ($BJ,<~Hf$,(B0$B$H$J$k$N$rKI;_$9$k(B)
  OutReg(UDLL, (UARTDivisor >> 0) & 0xff);
  OutReg(UDLM, (UARTDivisor >> 8) & 0xff);
  OutReg(ULCR, 0x03);		// data 8bit, stop 1bit, non-parity
  OutReg(UMCR, 0x03);		// RTS#, DTR# assert
  OutReg(UFCR, 0x07);		// FIFO$B;HMQ5v2D(B/FIFO$B%j%;%C%H(B

  return 0;
}

/* $BAw?.2DG=$+!)(B */
int serial_is_send_enable(int index)
{
  return InReg(ULSR) & 0x20;
}

/* $B#1J8;zAw?.(B */
int serial_send_byte(int index, unsigned char c)
{
  /* $BAw?.2DG=$K$J$k$^$GBT$D(B */
  while (!serial_is_send_enable(index));

  /* $BAw?.3+;O(B */
  OutReg(UTHR, c);

  return 0;
}

/* $B<u?.2DG=$+!)(B */
int serial_is_recv_enable(int index)
{
  return InReg(ULSR) & 0x1f;
}

/* $B#1J8;z<u?.(B */
unsigned char serial_recv_byte(int index)
{
  unsigned char s, c;

  while (1) {

    /* $B<u?.J8;z$,Mh$k$^$GBT$D(B */
    while (!(s = serial_is_recv_enable(index)));

    /* $B<u?.40N;(B */
    c = InReg(URBR);

    /* $B<u?.$7$?%G!<%?$K%(%i!<$,L5$$$3$H$r%A%'%C%/(B */
    if (!(s & 0x1e)) break;
  }

  return c;
}
